/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BASE_OSG_SCENE_GL_H_INCLUDED
#define CNOID_BASE_OSG_SCENE_GL_H_INCLUDED

#include <cnoid/OSGSceneObject>
#include <osg/Drawable>
#include "exportdecl.h"

namespace cnoid {

    class OSGSceneGLDrawable;

    /**
       This class is used for drawing a 3D object with raw OpenGL commands.
    */
    class CNOID_EXPORT OSGSceneGL : public OSGSceneObject
    {
      public:
        OSGSceneGL();
        OSGSceneGL(const OSGSceneGL& org, const osg::CopyOp& copyop = osg::CopyOp::SHALLOW_COPY);
        virtual ~OSGSceneGL();

        void dirtyDisplayList();
        
      protected:
        void setUseDisplayList(bool on);
        virtual void drawImplementation(osg::RenderInfo& ri) const = 0;
        virtual osg::BoundingBox computeGLBound() const;
        virtual void accept(osg::PrimitiveFunctor& functor) const;

      private:
        void initializeDrawable(OSGSceneGLDrawable* drawable);
        
        osg::ref_ptr<OSGSceneGLDrawable> drawable;

        friend class OSGSceneGLDrawable;
    };

    typedef osg::ref_ptr<OSGSceneGL> OSGSceneGLPtr;
}

#endif
