/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BASE_OSG_SCENE_VIEW_H_INCLUDED
#define CNOID_BASE_OSG_SCENE_VIEW_H_INCLUDED

#include <cnoid/View>
#include "exportdecl.h"

namespace osg {
    class Camera;
    class Node;
}

namespace cnoid {

    class ExtensionManager;
    class OSGViewer;
    class OSGSceneObject;
    class OSGSceneViewImpl;

    class CNOID_EXPORT OSGSceneView : public View
    {
      public:
        static void initialize(ExtensionManager* ext);
        static OSGSceneView* mainInstance();

        /**
           You can use OSGSceneObject's "show()" instead of this.
        */
        static void showSceneObject(OSGSceneObject* object, bool on);

        static void requestRedrawAll();

        OSGSceneView();
        virtual ~OSGSceneView();
        
        OSGViewer* viewer();

        /**
           Obsolete. Please use static function requestRedrawAll() or OSGSceneObject's requestRedraw()
        */
        void requestRedraw();
        
        /**
           Obsolete. Please use OSGSceneObject's show();
        */
        void addSceneObject(OSGSceneObject* object);

        /**
           Obsolete. Please use OSGSceneObject's hide();
        */
        void removeSceneObject(OSGSceneObject* object);

        void toggleFloorGrid(bool on);
        void toggleShadow(bool on);

        /**
           \todo Test to provide this flexible event signal
        */
        //SignalProxy<boost::signal<bool(OSGSceneObject* object, OSGSceneViewEvent* event)> > sigEvent;

        osg::Camera* getCameraControl();
        bool releaseCameraControl(osg::Camera* camera);

        void addPreprocessingNode(osg::Node* node);
        bool removePreprocessingNode(osg::Node* node);

        bool saveImage(const std::string& filename);

        void setScreenSize(int width, int height);

        void updateIndicator(const std::string& text);

      protected:

        virtual QWidget* indicatorOnInfoBar();
        virtual bool storeState(Archive& archive);
        virtual bool restoreState(const Archive& archive);

      private:
        friend class OSGSceneViewImpl;

        OSGSceneViewImpl* impl;
    };

}

#endif
