/**
   @author Shizuko Hattori
*/

#ifndef CNOID_BASE_SCENE_PIECES_H_INCLUDED
#define CNOID_BASE_SCENE_PIECES_H_INCLUDED

#include <cnoid/SceneGraph>
#include <cnoid/EigenTypes>
#include <GL/glew.h>
#include "GLSceneRenderer.h"
#include "exportdecl.h"

namespace cnoid {

    class CNOID_EXPORT SgCrossMarker : public SgCustomGLNode
    {
      public:
        EIGEN_MAKE_ALIGNED_OPERATOR_NEW

        SgCrossMarker(const Vector4& color, float size, float lineWidth = 1.0f);
        void setSize(float _size){ size = _size; };
        void setPosition(const Vector3& p);
        void renderCrossMarker();
      private:
        Vector3 position;
        Vector4 color;
        float size;
        float lineWidth;
    };
    typedef boost::intrusive_ptr<SgCrossMarker> SgCrossMarkerPtr;


    class CNOID_EXPORT SgSphereMarker : public SgTransform
    {
      public:
        SgSphereMarker(float radius, const Vector4& color);
        void setCross(const Vector4& color, float size, float lineWidth = 1.0f);
        void setPosition(const Vector3& p);
      private:
        SgCrossMarkerPtr cross;
    };
    typedef boost::intrusive_ptr<SgSphereMarker> SgSphereMarkerPtr;


    class CNOID_EXPORT SgBBMarker : public SgGroup
    {
      public:
        SgBBMarker(const SgBoundingBox& bbox, const Vector4& color, float width);
        SgBBMarker(const SgBoundingBox& bbox, const Vector4& color);
        
      private:
        void create(const SgBoundingBox& bbox, const Vector4& color, float width);
        void addMarker(double x, double y, double z, SgShapePtr shape);
    };
    typedef boost::intrusive_ptr<SgBBMarker> SgBBMarkerPtr;


    class CNOID_EXPORT SgAttitudeDragger : public SgTransform
    {
      public:
        SgAttitudeDragger();
        void attachTo(SgTransformPtr parent);
        inline void detach() { attachTo(0); }
        inline float radius() { return radius_; }
        
      private:
        SgShapePtr draggers[3];
        SgGroup* currentParent;
        float radius_;
    };
    typedef boost::intrusive_ptr<SgAttitudeDragger> SgAttitudeDraggerPtr;
}

#endif
