/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BASE_SCENE_VIEW_H_INCLUDED
#define CNOID_BASE_SCENE_VIEW_H_INCLUDED

#include <cnoid/View>
#include <cnoid/SceneGraph>
#include "exportdecl.h"

namespace cnoid {

    class SceneViewImpl;
    class SgNode;
    typedef boost::intrusive_ptr<SgNode> SgNodePtr;

    class CNOID_EXPORT SceneView : public View
    {
    public:
        static void initialize(ExtensionManager* ext);
        static SceneView* instance();

        SceneView();
        ~SceneView();

        void addEntity(SgNodePtr entity);
        void removeEntity(SgNodePtr entity);

        bool saveImage(const std::string& filename);
        QImage getImage();
        void setScreenSize(int width, int height);

        /**
           \todo use a signal of SceneObject to show messages in the indicator
        */
        void updateIndicator(const std::string& text);

      protected:

        virtual void onActivated();
        virtual void onDeactivated();
        virtual QWidget* indicatorOnInfoBar();
        virtual bool storeState(Archive& archive);
        virtual bool restoreState(const Archive& archive);

      private:
        SceneViewImpl* impl;
    };
}

#endif
