/*!
  @file
  @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODYPLUGIN_AIST_SIMULATOR_ITEM_H_INCLUDED
#define CNOID_BODYPLUGIN_AIST_SIMULATOR_ITEM_H_INCLUDED

#include "SimulatorItem.h"

namespace cnoid {

    class AISTSimulatorItemImpl;
        
    class AISTSimulatorItem : public SimulatorItem
    {
      public:

        static void initialize(ExtensionManager* ext);
        
        AISTSimulatorItem();
        AISTSimulatorItem(const AISTSimulatorItem& org);
        virtual ~AISTSimulatorItem();
        
      protected:
        
        virtual SimulationBodyPtr createSimulationBody(const Body& orgBody);
        virtual ControllerItem* createBodyMotionController(BodyItem* bodyItem, BodyMotionItem* bodyMotionItem);
        virtual bool initializeSimulation(const std::vector<SimulationBodyPtr>& simBodies);
        virtual bool stepSimulation(const std::vector<SimulationBodyPtr>& activeSimBodies);
        virtual void finalizeSimulation();
        
        virtual ItemPtr doDuplicate() const;
        virtual void doPutProperties(PutPropertyFunction& putProperty);
        virtual bool store(Archive& archive);
        virtual bool restore(const Archive& archive);

      private:
        AISTSimulatorItemImpl* impl;
        friend class AISTSimulatorItemImpl;
    };

    typedef boost::intrusive_ptr<AISTSimulatorItem> AISTSimulatorItemPtr;
}

#endif
