/**
   @file
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODYPLUGIN_BODY_MOTION_ITEM_H_INCLUDED
#define CNOID_BODYPLUGIN_BODY_MOTION_ITEM_H_INCLUDED

#include <cnoid/BodyMotion>
#include <cnoid/MultiValueSeqItem>
#include <cnoid/MultiSE3SeqItem>
#include <cnoid/Vector3SeqItem>
#include "exportdecl.h"

namespace cnoid {

    class BodyItemStateChanger;
    typedef boost::shared_ptr<BodyItemStateChanger> BodyItemStateChangerPtr;

    class CNOID_EXPORT BodyMotionItem : public AbstractMultiSeqItem
    {
      public:

        static void initialize(ExtensionManager* ext);

        static void addExtraSeqItemFactory(
            const std::string& key, boost::function<AbstractSeqItemPtr(AbstractSeqPtr seq)> factory);

        BodyMotionItem();
        BodyMotionItem(BodyMotionPtr bodyMotion);
        ~BodyMotionItem();

        virtual AbstractMultiSeqPtr abstractMultiSeq();

        inline const BodyMotionPtr& motion() { return bodyMotion_; }

        inline MultiValueSeqItem* jointPosSeqItem() {
            return jointPosSeqItem_.get();
        }

        inline const MultiValueSeqPtr& jointPosSeq() {
            return bodyMotion_->jointPosSeq();
        }

        inline MultiSE3SeqItem* linkPosSeqItem() {
            return linkPosSeqItem_.get();
        }
            
        inline const MultiSE3SeqPtr& linkPosSeq() {
            return bodyMotion_->linkPosSeq();
        }

        virtual void notifyUpdate();

      private:
        struct ExtraSeqItemInfo {
            operator AbstractSeqItemPtr() const { return item; }
            AbstractSeqItem* operator->() { return item.get(); }
            AbstractSeqItem& operator*() { return *item; }
        private:
            ExtraSeqItemInfo(AbstractSeqItemPtr& item) : item(item) { }
            AbstractSeqItemPtr item;
            boost::signals::connection sigUpdateConnection;
            friend class BodyMotionItem;
        };
        typedef std::map<std::string, ExtraSeqItemInfo> ExtraSeqItemMap;

      public:
        typedef ExtraSeqItemInfo ExtraSeqItem;
        typedef ExtraSeqItemMap::const_iterator ConstSeqIterator;

        ConstSeqIterator extraSeqItemBegin() const { return extraSeqItems.begin(); }
        ConstSeqIterator extraSeqItemEnd() const { return extraSeqItems.end(); }

        SignalProxy< boost::signal<void()> > sigExtraSeqItemsChanged() {
            return sigExtraSeqItemsChanged_;
        }

        void updateExtraSeqItems();

      protected:

        BodyMotionItem(const BodyMotionItem& org);

        virtual bool onNewChildItem(Item* childItem, bool isManualOperation);
        virtual ItemPtr doDuplicate() const;
        virtual bool store(Archive& archive);
        virtual bool restore(const Archive& archive);

      private:

        BodyMotionPtr bodyMotion_;
            
        MultiValueSeqItemPtr jointPosSeqItem_;
        MultiSE3SeqItemPtr linkPosSeqItem_;
        Vector3SeqItemPtr ZMPseqItem_;

        ExtraSeqItemMap extraSeqItems;
        boost::signal<void()> sigExtraSeqItemsChanged_;
        boost::signals::connection sigExtraSeqsChangedConnection;

        void initialize();
        void onExtraSeqItemSetChanged();
        void onSubItemUpdated();
    };

    typedef boost::intrusive_ptr<BodyMotionItem> BodyMotionItemPtr;
}

#endif
