/**
   @author 
*/

#ifndef CNOID_OPENRTM_PLUGIN_RTC_ITEM_H_INCLUDED
#define CNOID_OPENRTM_PLUGIN_RTC_ITEM_H_INCLUDED

#include <cnoid/Item>
#include <cnoid/Process>
#include <rtm/Manager.h>

using namespace std;
using namespace RTC;

namespace cnoid {

    class MessageView;
    typedef map<string, string> PropertyMap;

    class RTComponent
    {
    public:
        RTComponent(string moduleName);
        RTComponent(string moduleName, PropertyMap& properties);
        ~RTComponent();
        void deleteRTC(bool waitToBeDeleted);
        RtcBase* rtc() { return rtc_; };
        bool isValid() const;
        const std::string& name() const { return componentName; }
        
    private:
        RTObject_var rtcRef;
        RtcBase* rtc_;
        string moduleName;
        Process rtcProcess;
        string componentName;
        MessageView* mv;

        void init(string moduleName, PropertyMap& properties);
        bool createRTC(PropertyMap& properties);
        void setupModules(string& fileName, string& initFuncName, string& componentName, PropertyMap& properties );
        void createProcess(string& command, PropertyMap& properties);
        void onReadyReadServerProcessOutput();
    };

    class RTCItem : public Item
    {
    public:
        static void initialize(ExtensionManager* ext);
        
        RTCItem();
        RTCItem(const RTCItem& org);
        virtual ~RTCItem();
               
    protected:
        virtual void onPositionChanged();
        virtual void onDisconnectedFromRoot();
        virtual ItemPtr doDuplicate() const;
        virtual void doPutProperties(PutPropertyFunction& putProperty);
        virtual bool store(Archive& archive);
        virtual bool restore(const Archive& archive);
        
    private:
        ostream& os;
        MessageView* mv;
        string moduleName;
        RTComponent* rtcomp;
        Selection periodicType;
        int periodicRate;
        int oldType;
        PropertyMap properties;

        void setModuleName(const std::string& name);
        void setPeriodicType(const int type);
        void setPeriodicRate(const int rate);
    };
        
    typedef boost::intrusive_ptr<RTCItem> RTCItemPtr;
}

#endif
