/*!
  @file
  @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_UTIL_FILE_UTIL_H_INCLUDED
#define CNOID_UTIL_FILE_UTIL_H_INCLUDED

#include <boost/filesystem.hpp>
#include <string>
#include "exportdecl.h"

namespace cnoid {

    CNOID_EXPORT void makePathCompact(
        const boost::filesystem::path& path,
        boost::filesystem::path& out_compact);

    CNOID_EXPORT int findSubDirectory(
        const boost::filesystem::path& directory,
        const boost::filesystem::path& path,
        boost::filesystem::path& out_subdirectory);

    CNOID_EXPORT bool findRelativePath(
        const boost::filesystem::path& from,
        const boost::filesystem::path& to,
        boost::filesystem::path& out_relativePath);


    // for the boost filesystem compatibility
    
#if BOOST_FILESYSTEM_VERSION == 3

    inline std::string getGenericPathString(const boost::filesystem::path& path) {
        return path.generic_string();
    }
    inline bool checkAbsolute(const boost::filesystem::path& path){
        return path.is_absolute();
    }
    inline boost::filesystem::path getAbsolutePath(const boost::filesystem::path& path){
        return boost::filesystem::absolute(path);
    }
    inline std::string getAbsolutePathString(const boost::filesystem::path& path){
        return boost::filesystem::absolute(path).string();
    }
    inline std::string getFilename(const boost::filesystem::path& path) {
        return path.filename().string();
    }
    inline std::string getBasename(const boost::filesystem::path& path) {
        return path.stem().string();
    }
    inline std::string getPathString(const boost::filesystem::path& path) {
        return path.string();
    }
    inline std::string getNativePathString(const boost::filesystem::path& path) {
        boost::filesystem::path p(path);
        return p.make_preferred().string();
    }

#else
    inline std::string getGenericPathString(const boost::filesystem::path& path) {
        return path.string();
    }
    inline bool checkAbsolute(const boost::filesystem::path& path){
        return path.is_complete();
    }
    inline boost::filesystem::path getAbsolutePath(const boost::filesystem::path& path){
        return boost::filesystem::complete(path);
    }
    inline std::string getAbsolutePathString(const boost::filesystem::path& path){
        return boost::filesystem::complete(path).file_string();
    }
    inline std::string getFilename(const boost::filesystem::path& path) {
        return path.filename();
    }
    inline std::string getBasename(const boost::filesystem::path& path) {
        return path.stem();
    }
    inline std::string getPathString(const boost::filesystem::path& path) {
        return path.file_string();
    }
    inline std::string getNativePathString(const boost::filesystem::path& path) {
        return path.file_string();
    }
#endif



//#ifdef _WIN32
#if 0
    CNOID_EXPORT const std::string toActualPathName(const std::string& pathName);
#else
    inline const std::string& toActualPathName(const std::string& pathName) {
        return pathName;
    }
#endif
    
}

#endif
