/**
   @author Shin'ichiro Nakaoka
*/

#ifndef CNOID_UTIL_REFERENCED_H_INCLUDED
#define CNOID_UTIL_REFERENCED_H_INCLUDED

#include <boost/intrusive_ptr.hpp>

#if (BOOST_VERSION >= 105300)
#include <boost/atomic.hpp>
#define CNOID_REFERENCED_USE_ATOMIC_COUNTER
#endif


namespace cnoid {
    class Referenced;
}

namespace cnoid{
    void intrusive_ptr_add_ref(cnoid::Referenced* obj);
    void intrusive_ptr_release(cnoid::Referenced* obj);
}

namespace cnoid {

    /**
       \todo make a pointer to this object thread safe
    */
    class Referenced
    {
    public:
        inline Referenced() { refCount_ = 0; }
        virtual ~Referenced() { }

#ifdef CNOID_REFERENCED_USE_ATOMIC_COUNTER
    private:
        mutable boost::atomic<int> refCount_;

    protected:
        int refCount() const { return refCount_.load(boost::memory_order_relaxed); }
#else
    private:
        int refCount_;
    protected:
        int refCount() const { return refCount_; }
#endif
        friend void cnoid::intrusive_ptr_add_ref(Referenced* obj);
        friend void cnoid::intrusive_ptr_release(Referenced* obj);
    };

    typedef boost::intrusive_ptr<Referenced> ReferencedPtr;
}


namespace cnoid
{
#ifdef CNOID_REFERENCED_USE_ATOMIC_COUNTER    
    inline void intrusive_ptr_add_ref(const cnoid::Referenced* obj) {
        obj->refcount_.fetch_add(1, boost::memory_order_relaxed);
    }

    inline void intrusive_ptr_release(const cnoid::Referenced* obj){
        if(obj->refcount_.fetch_sub(1, boost::memory_order_release) == 1) {
            boost::atomic_thread_fence(boost::memory_order_acquire);
            delete obj;
        }
    }
#else        
    inline void intrusive_ptr_add_ref(cnoid::Referenced* obj){
        obj->refCount_++;
    }

    inline void intrusive_ptr_release(cnoid::Referenced* obj){
        obj->refCount_--;
        if(obj->refCount_ == 0){
            delete obj;
        }
    }
#endif
};

#endif
